  /* verbose syntax error messages */
%define parse.error verbose
  /* pure (reentrant) parser */
%define api.pure full
  /* handle locations for nice error messages */
%locations


  /* generate a header */
%defines "calc.h"


%{

#include <stdio.h>
#include "calc.h"

void yyerror(YYLTYPE* yyloc, const char*);

%}

%code provides {

#define YY_DECL \
  enum yytokentype yylex(YYSTYPE* yylval, YYLTYPE* yyloc)
YY_DECL;

}


%union
{
  double val;
}

%token<val> INT
%token<val> FLOAT
%token PLUS
%token MINUS
%token TIMES
%token DIV
%token LPAR
%token RPAR

%token EOL

%left PLUS MINUS
%left TIMES DIV

%type<val> exp

  /* Indicate the starting non-terminal */
%start lines

%%

lines:
        lines line
     |  %empty
     ;

line:
        exp EOL         { printf("%f\n", $1); }
      ;

exp:
      exp PLUS exp      { $$ = $1+$3; }
    | exp MINUS exp     { $$ = $1-$3; }
    | exp TIMES exp     { $$ = $1*$3; }
    | exp DIV exp       {
                          if ($3)
                            $$ = $1/$3;
                          else
                          {
                            yyerror(&yylloc, "Cannot divide by zero!");
                            YYERROR;
                          }
                        }
    | LPAR exp RPAR     { $$=$2; }
    | INT               { $$=$1; }
    | FLOAT             { $$=$1; }
    ;

%%

void yyerror(YYLTYPE* yyloc, const char* msg)
{
  fprintf(stderr, "at line %d, column %d\n", yyloc->first_line, yyloc->first_column);
  fprintf(stderr, "%s\n", msg);
}

int main(void)
{
  int r = yyparse();
  return r;
}
